################################################################################
#
# Prozeduren zur Erzeugung von BoxRows fuer die Eingabe und Pruefung von 
# mathematischen Objekten
#
################################################################################


with(StringTools):

################################################################################
#
# BoxRow fuer lineare Gleichungssystem
#
################################################################################

create_boxrow_lgs:=proc(Ueberschrift, ListeGleichungsbezeichnungen, SchalterML, NameKorrektesSystem)

#  Ueberschrift: String, der die Ueberschrift ueber die Gleichungsbox enthaelt
#  ListeGleichungsbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 Namen der Gleichungen enthlt (z.B. "Gleichgewicht in 
#				 x-Richtung"); soll keine Name erscheinen, wird ein leerer
#				 String ("") verwendet.
#  SchalterML: boolean; true, falls Umwandlung in MathML-Schoenschrift angeboten werden soll
#	       sonst false.
#  NameKorrektesSystem: Name deskorrekten Systems (wird zum Checken benoetigt).
#
# Rueckgabe: String mit entsprechender BoxRow-Spezifikation

local	rueck_string, i, SystemTextFieldBezug, GleichungsfelderTextFieldBezug, gleichungsfeld, \
	gleichungsfeldname, zufallsname, mathmlfeld, mathmlfeldname;

rueck_string:=StringBuffer():

rueck_string:=rueck_string:-append("\tBoxRow( \n \t\t\
border=true, hscroll=as_needed,  vscroll=as_needed,\n");

SystemTextFieldBezug:=cat("A",convert(rand(),string));
GleichungsfelderTextFieldBezug:=cat("A",convert(rand(),string));

for i from 1 to nops(ListeGleichungsbezeichnungen) do
zufallsname:=convert(rand(),string);
gleichungsfeldname[i]:=cat("A",zufallsname,"name");
gleichungsfeld[i]:=cat("A",zufallsname);
mathmlfeldname[i]:=cat("ML",zufallsname,"name");
mathmlfeld[i]:=cat("ML",zufallsname);
end do;


rueck_string:=rueck_string:-append("\t\t[BoxRow('border'=false, visible=false, TextField['");
rueck_string:=rueck_string:-append(SystemTextFieldBezug);
rueck_string:=rueck_string:-append("'](value=\"");
rueck_string:=rueck_string:-append(NameKorrektesSystem);
rueck_string:=rueck_string:-append("\", visible=false, width=1), \n\t\t\tTextField['");
					  
rueck_string:=rueck_string:-append(GleichungsfelderTextFieldBezug);
rueck_string:=rueck_string:-append("'](value=\"{");
for i from 1 to nops(ListeGleichungsbezeichnungen) do
	rueck_string:=rueck_string:-append(gleichungsfeld[i]);
	if i<nops(ListeGleichungsbezeichnungen) then
		rueck_string:=rueck_string:-append(",");
	end if;
end do;

rueck_string:=rueck_string:-append("}\", visible=false,width=1)),\n");

if length(Ueberschrift)>0 then
	rueck_string:=rueck_string:-append("\n\t\tBoxRow(Label('font' = Font(\"arial\", bold, 12),\"");
	rueck_string:=rueck_string:-append(Ueberschrift);
	rueck_string:=rueck_string:-append("\")),");
end if;

for i from 1 to nops(ListeGleichungsbezeichnungen) do
	rueck_string:=rueck_string:-append("\n\t\tBoxRow(TextField['");
	rueck_string:=rueck_string:-append(gleichungsfeldname[i]);
	rueck_string:=rueck_string:-append("'](value=\"");
	rueck_string:=rueck_string:-append(gleichungsfeld[i]);
	rueck_string:=rueck_string:-append("\", visible=false,width=1),");

	rueck_string:=rueck_string:-append("\n\t\tTextField['");
	rueck_string:=rueck_string:-append(mathmlfeldname[i]);
	rueck_string:=rueck_string:-append("'](value=\"");
	rueck_string:=rueck_string:-append(mathmlfeld[i]);
	rueck_string:=rueck_string:-append("\", visible=false,width=1),\"");

	rueck_string:=rueck_string:-append(ListeGleichungsbezeichnungen[i]);
	rueck_string:=rueck_string:-append("\"\n\t\t, TextField['");
	rueck_string:=rueck_string:-append(gleichungsfeld[i]);
	rueck_string:=rueck_string:-append("'](25, 'font' = Font(\"arial\",12)),");

	if SchalterML=true then
		rueck_string:=rueck_string:-append("\n\t\tButton(\">>schn>>\",  \n\t\t\t\
Evaluate(function=\"umwandeln\",Argument('");
		rueck_string:=rueck_string:-append(gleichungsfeldname[i]);
		rueck_string:=rueck_string:-append("'),Argument('");
		rueck_string:=rueck_string:-append(mathmlfeldname[i]);
		rueck_string:=rueck_string:-append("'))),");
		rueck_string:=rueck_string:-append("\n\t\tMathMLViewer['");
		rueck_string:=rueck_string:-append(mathmlfeld[i]);
		rueck_string:=rueck_string:-append("'](height=50),\n");
	end if;
	rueck_string:=rueck_string:-append("\n\t\tButton(\"Check Gleichung\",  \n\t\t\t\
Evaluate(function=\"check_gleichung\",Argument('");
	rueck_string:=rueck_string:-append(gleichungsfeld[i]);
	rueck_string:=rueck_string:-append("'),Argument('");
	rueck_string:=rueck_string:-append(SystemTextFieldBezug);
	rueck_string:=rueck_string:-append("'))))");

	rueck_string:=rueck_string:-append(",");
	rueck_string:=rueck_string:-append("\n\n");
end do;


rueck_string:=rueck_string:-append("\n\t\tBoxRow(Button(\"Checke System\",\n\
\t\tEvaluate(function=\"check_system\",Argument('");
rueck_string:=rueck_string:-append(GleichungsfelderTextFieldBezug);
rueck_string:=rueck_string:-append("'),Argument('");
rueck_string:=rueck_string:-append(SystemTextFieldBezug);
rueck_string:=rueck_string:-append("'))),\n\t\t");
rueck_string:=rueck_string:-append("Button(\"Zeige fehlende Gleichung\",\n\t\t\
Evaluate(function=\"zeige_fehlende_gleichung\",Argument('");
rueck_string:=rueck_string:-append(GleichungsfelderTextFieldBezug);
rueck_string:=rueck_string:-append("'),Argument('");
rueck_string:=rueck_string:-append(SystemTextFieldBezug);
rueck_string:=rueck_string:-append("')))),\n\t\t");

rueck_string:=rueck_string:-append("\n\t\tBoxRow(Button(\"Lse System\",\n\
\t\tEvaluate(function=\"loese_system\",Argument('");
rueck_string:=rueck_string:-append(GleichungsfelderTextFieldBezug);
rueck_string:=rueck_string:-append("'),Argument('");
rueck_string:=rueck_string:-append(SystemTextFieldBezug);
rueck_string:=rueck_string:-append("'))),\n\t\t");
rueck_string:=rueck_string:-append("Button(\"Zeige korrektes System\",\n\t\t\
Evaluate(function=\"zeige_korrektes_system\",Argument('");
rueck_string:=rueck_string:-append(GleichungsfelderTextFieldBezug);
rueck_string:=rueck_string:-append("'),Argument('");
rueck_string:=rueck_string:-append(SystemTextFieldBezug);
rueck_string:=rueck_string:-append("'))))]\n\n");

rueck_string:=rueck_string:-append("\t)\n");

return rueck_string:-value();

end:


################################################################################
#
# BoxRow fuer Ausdruecke
#
################################################################################

create_boxrow_ausdruecke:=proc(Ueberschrift, ListeAusdrucksbezeichnungen, SchalterML, \
				ListeNameKorrekteAusdruecke)

#  Ueberschrift: String, der die Ueberschrift ueber die Ausdrucksbox enthaelt
#  ListeAusdrucksbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 Namen der Ausdruecke enthlt (z.B. "Gleichgewicht in 
#				 x-Richtung").
#  SchalterML: boolean; true, falls Umwandlung in MathML-Schoenschrift angeboten werden soll
#	       sonst false.
#  ListeNameKorrekteAusdruecke: Liste mit Namen der korrekten Ausdruecke (wird zum Checken benoetigt).
#
# Rueckgabe: String mit entsprechender BoxRow-Spezifikation

local	rueck_string, i, ausdrucksfeld, \
	ausdrucksfeldname, korrekterausdrucksfeldname, zufallsname, mathmlfeld, mathmlfeldname;

rueck_string:=StringBuffer():

rueck_string:=rueck_string:-append("\tBoxRow( \n \t\t\
border=true, hscroll=as_needed,  vscroll=as_needed,\n");


for i from 1 to nops(ListeAusdrucksbezeichnungen) do
zufallsname:=convert(rand(),string);
ausdrucksfeldname[i]:=cat("A",zufallsname,"name");
ausdrucksfeld[i]:=cat("A",zufallsname);
korrekterausdrucksfeldname[i]:=cat("KA",zufallsname);
mathmlfeldname[i]:=cat("ML",zufallsname,"name");
mathmlfeld[i]:=cat("ML",zufallsname);
end do;


if length(Ueberschrift)>0 then
	rueck_string:=rueck_string:-append("\n\t\t[BoxRow(Label('font' = Font(\"arial\", bold, 12),\"");
	rueck_string:=rueck_string:-append(Ueberschrift);
	rueck_string:=rueck_string:-append("\")),");
else 	rueck_string:=rueck_string:-append("\n\t\t[");
end if;

for i from 1 to nops(ListeAusdrucksbezeichnungen) do
	rueck_string:=rueck_string:-append("\n\t\tBoxRow(TextField['");
	rueck_string:=rueck_string:-append(ausdrucksfeldname[i]);
	rueck_string:=rueck_string:-append("'](value=\"");
	rueck_string:=rueck_string:-append(ausdrucksfeld[i]);
	rueck_string:=rueck_string:-append("\", visible=false,width=1),");

	rueck_string:=rueck_string:-append("\n\t\tTextField['");
	rueck_string:=rueck_string:-append(korrekterausdrucksfeldname[i]);
	rueck_string:=rueck_string:-append("'](value=\"");
	rueck_string:=rueck_string:-append(ListeNameKorrekteAusdruecke[i]);
	rueck_string:=rueck_string:-append("\", visible=false,width=1),");

	rueck_string:=rueck_string:-append("\n\t\tTextField['");
	rueck_string:=rueck_string:-append(mathmlfeldname[i]);
	rueck_string:=rueck_string:-append("'](value=\"");
	rueck_string:=rueck_string:-append(mathmlfeld[i]);
	rueck_string:=rueck_string:-append("\", visible=false,width=1),\"");

	rueck_string:=rueck_string:-append(ListeAusdrucksbezeichnungen[i]);
	rueck_string:=rueck_string:-append("\"\n\t\t, TextField['");
	rueck_string:=rueck_string:-append(ausdrucksfeld[i]);
	rueck_string:=rueck_string:-append("'](25, 'font' = Font(\"arial\",12)),");

	if SchalterML=true then
		rueck_string:=rueck_string:-append("\n\t\tButton(\">>schn>>\",  \n\t\t\t\
Evaluate(function=\"umwandeln\",Argument('");
		rueck_string:=rueck_string:-append(ausdrucksfeldname[i]);
		rueck_string:=rueck_string:-append("'),Argument('");
		rueck_string:=rueck_string:-append(mathmlfeldname[i]);
		rueck_string:=rueck_string:-append("'))),");
		rueck_string:=rueck_string:-append("\n\t\tMathMLViewer['");
		rueck_string:=rueck_string:-append(mathmlfeld[i]);
		rueck_string:=rueck_string:-append("'](height=50),\n");
	end if;
	rueck_string:=rueck_string:-append("\n\t\tButton(\"Check Ausdruck\",  \n\t\t\t\
Evaluate(function=\"check_ausdruck\",Argument('");
	rueck_string:=rueck_string:-append(ausdrucksfeld[i]);
	rueck_string:=rueck_string:-append("'),Argument('");
	rueck_string:=rueck_string:-append(korrekterausdrucksfeldname[i]);
	rueck_string:=rueck_string:-append("'))))");
	if i<nops(ListeAusdrucksbezeichnungen) then
		rueck_string:=rueck_string:-append(",");
	else 	rueck_string:=rueck_string:-append("]");
	end if;
	rueck_string:=rueck_string:-append("\n\n");
end do;



rueck_string:=rueck_string:-append("\t)\n");

return rueck_string:-value();

end:


################################################################################
#
# BoxRow fuer Vektoren
#
################################################################################

create_boxrow_vektoren:=proc(Ueberschrift, ListeVektorbezeichnungen, SchalterML, \
				ListeNameKorrekteVektoren)

#  Ueberschrift: String, der die Ueberschrift ueber die Vektorenbox enthaelt
#  ListeVektorbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 Namen der Vektoren enthlt (z.B. "F").
#  SchalterML: boolean; true, falls Umwandlung in MathML-Schoenschrift angeboten werden soll
#	       sonst false. Wird momentan nicht angeboten.
#  ListeNameKorrekteVektoren: Liste mit Namen der korrekten Vektoren (wird zum Checken benoetigt).
#
# Rueckgabe: String mit entsprechender BoxRow-Spezifikation

local	rueck_string, i, vektorfeld, \
	ListeFelderKorrekteVektoren, zufallsname;

rueck_string:=StringBuffer():

rueck_string:=rueck_string:-append("\tBoxRow( \n \t\t\
border=true, hscroll=as_needed,  vscroll=as_needed,\n");


for i from 1 to nops(ListeVektorbezeichnungen) do
zufallsname:=convert(rand(),string);
ListeFelderKorrekteVektoren[i]:=cat(ListeNameKorrekteVektoren[i],zufallsname);
vektorfeld[i]:=cat("A",zufallsname);
end do;


if length(Ueberschrift)>0 then
	rueck_string:=rueck_string:-append("\n\t\t[BoxRow(Label('font' = Font(\"arial\", bold, 12),\"");
	rueck_string:=rueck_string:-append(Ueberschrift);
	rueck_string:=rueck_string:-append("\")),");
else 	rueck_string:=rueck_string:-append("\n\t\t[");
end if;

rueck_string:=rueck_string:-append("\n\t\tBoxRow(visible=false");
for i from 1 to nops(ListeVektorbezeichnungen) do
	rueck_string:=rueck_string:-append("\n\t\t, TextField['");
	rueck_string:=rueck_string:-append(ListeFelderKorrekteVektoren[i]);
	rueck_string:=rueck_string:-append("'](value=\"");
	rueck_string:=rueck_string:-append(ListeNameKorrekteVektoren[i]);
	rueck_string:=rueck_string:-append("\", visible=false,width=1)\n");
end do;
rueck_string:=rueck_string:-append("\t\t),");

rueck_string:=rueck_string:-append("\n\t\tGridLayout('border'=false,\n");
rueck_string:=rueck_string:-append("\t\t[ [\"Name\",\"x-Koordinate\",\"y-Koordinate\",\"z-Koordinate\",\"\"],\n");
for i from 1 to nops(ListeVektorbezeichnungen) do

	rueck_string:=rueck_string:-append("\n\t\t[\"");

	rueck_string:=rueck_string:-append(ListeVektorbezeichnungen[i]);
	rueck_string:=rueck_string:-append("\", TextField['");
	rueck_string:=rueck_string:-append(vektorfeld[i]);
	rueck_string:=rueck_string:-append("X'](width=10),");
	rueck_string:=rueck_string:-append(" TextField['");
	rueck_string:=rueck_string:-append(vektorfeld[i]);
	rueck_string:=rueck_string:-append("Y'](width=10),");
	rueck_string:=rueck_string:-append(" TextField['");
	rueck_string:=rueck_string:-append(vektorfeld[i]);
	rueck_string:=rueck_string:-append("Z'](width=10),");


	rueck_string:=rueck_string:-append("\n\t\tButton(\"Check Vektor\",  \n\t\t\t\
Evaluate(function=\"check_vector\",Argument('");
	rueck_string:=rueck_string:-append(vektorfeld[i]);
	rueck_string:=rueck_string:-append("X'),Argument('");
	rueck_string:=rueck_string:-append(vektorfeld[i]);
	rueck_string:=rueck_string:-append("Y'),Argument('");
	rueck_string:=rueck_string:-append(vektorfeld[i]);
	rueck_string:=rueck_string:-append("Z'),Argument('");
	rueck_string:=rueck_string:-append(ListeFelderKorrekteVektoren[i]);
	rueck_string:=rueck_string:-append("')))]");
	if i<nops(ListeVektorbezeichnungen) then
		rueck_string:=rueck_string:-append(",");
	else 	rueck_string:=rueck_string:-append("])");
	end if;
end do;

rueck_string:=rueck_string:-append("\n\t\t]\n");

rueck_string:=rueck_string:-append("\t)\n");

return rueck_string:-value();

end:


################################################################################
#
# BoxRow fuer Matrizen
#
################################################################################

create_boxrow_matrix:=proc(Ueberschrift, ListeZeilenbezeichnungen, ListeSpaltenbezeichnungen, SchalterML, \
				NameKorrekteMatrix)

#  Ueberschrift: String, der die Ueberschrift ueber die Matrixbox enthaelt
#  ListeZeilenbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 Namen der Zeilen enthlt (z.B. "F").
#  ListeSpaltenbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 Namen der Spalten enthlt (z.B. "F").
#  SchalterML: boolean; true, falls Umwandlung in MathML-Schoenschrift angeboten werden soll
#	       sonst false. Wird momentan nicht angeboten.
#  NameKorrekteMatrix: String, der den Namen der korrekten Matrix enthaelt (wird zum Checken benoetigt).
#
# Rueckgabe: String mit entsprechender BoxRow-Spezifikation

local	rueck_string, i, j, zufallsnamematrix,zufallsnamezeilen,zufallsnamespalten, \
	zeilenstring, spaltenstring, zufallsname, spaltenstringohne;

rueck_string:=StringBuffer():

rueck_string:=rueck_string:-append("\tBoxRow( \n \t\t\
border=true, hscroll=as_needed,  vscroll=as_needed,\n");

zufallsname:=convert(rand(),string);
zufallsnamematrix:=cat("Matrix",zufallsname);
zufallsnamezeilen:=cat("Zeilen",zufallsname);
zufallsnamespalten:=cat("Spalten",zufallsname);

if length(Ueberschrift)>0 then
	rueck_string:=rueck_string:-append("\n\t\t[BoxRow(Label('font' = Font(\"arial\", bold, 12),\"");
	rueck_string:=rueck_string:-append(Ueberschrift);
	rueck_string:=rueck_string:-append("\")),");
else 	rueck_string:=rueck_string:-append("\n\t\t[");
end if;

rueck_string:=rueck_string:-append("\n\t\tBoxRow(visible=false");

rueck_string:=rueck_string:-append("\n\t\t, TextField['");
rueck_string:=rueck_string:-append(zufallsnamematrix);
rueck_string:=rueck_string:-append("'](value=\"");
rueck_string:=rueck_string:-append(NameKorrekteMatrix);
rueck_string:=rueck_string:-append("\", visible=false,width=1)\n");

rueck_string:=rueck_string:-append("\n\t\t, TextField['");
rueck_string:=rueck_string:-append(zufallsnamezeilen);
rueck_string:=rueck_string:-append("'](value=\"");
zeilenstring:="[";
for i from 1 to nops(ListeZeilenbezeichnungen)-1 do
	zeilenstring:=cat(zeilenstring,ListeZeilenbezeichnungen[i],zufallsname, ",");
end do;
zeilenstring:=cat(zeilenstring,ListeZeilenbezeichnungen[nops(ListeZeilenbezeichnungen)],zufallsname,"]");
rueck_string:=rueck_string:-append(zeilenstring);
rueck_string:=rueck_string:-append("\", visible=false,width=1)\n");

rueck_string:=rueck_string:-append("\n\t\t, TextField['");
rueck_string:=rueck_string:-append(zufallsnamespalten);
rueck_string:=rueck_string:-append("'](value=\"");
spaltenstring:="[";	spaltenstringohne:="[";
for i from 1 to nops(ListeSpaltenbezeichnungen)-1 do
	spaltenstring:=cat(spaltenstring,ListeSpaltenbezeichnungen[i],zufallsname,",");
	spaltenstringohne:=cat(spaltenstringohne,ListeSpaltenbezeichnungen[i],",");
end do;
spaltenstring:=cat(spaltenstring,ListeSpaltenbezeichnungen[nops(ListeSpaltenbezeichnungen)],zufallsname,"]");
spaltenstringohne:=cat(spaltenstringohne,ListeSpaltenbezeichnungen[nops(ListeSpaltenbezeichnungen)],"]");
rueck_string:=rueck_string:-append(spaltenstring);
rueck_string:=rueck_string:-append("\", visible=false,width=1)\n");

rueck_string:=rueck_string:-append("\n\t\t, TextField['");
rueck_string:=rueck_string:-append("spaltennamenfeld"):-append(zufallsname);
rueck_string:=rueck_string:-append("'](value=\"");
rueck_string:=rueck_string:-append(spaltenstringohne);
rueck_string:=rueck_string:-append("\", visible=false,width=1)\n");


rueck_string:=rueck_string:-append("\n\t\t),");


rueck_string:=rueck_string:-append("\n\t\tGridLayout('border'=false,\n");
rueck_string:=rueck_string:-append("\t\t[ [\"\","); 
for i from 1 to nops(ListeSpaltenbezeichnungen) do
	rueck_string:=rueck_string:-append("\""):-append(ListeSpaltenbezeichnungen[i]):-append("\"");
	if i<nops(ListeSpaltenbezeichnungen) then
		rueck_string:=rueck_string:-append(",");
	else
		rueck_string:=rueck_string:-append("],");
	end if;
end do;

for i from 1 to nops(ListeZeilenbezeichnungen) do
	rueck_string:=rueck_string:-append("\t\t[ \""):-append(ListeZeilenbezeichnungen[i]):-append("\",");
	for j from 1 to nops(ListeSpaltenbezeichnungen) do
		rueck_string:=rueck_string:-append("TextField['");
		rueck_string:=rueck_string:-append(ListeSpaltenbezeichnungen[j])\
					  :-append(zufallsname)\
					  :-append(ListeZeilenbezeichnungen[i])\
					  :-append(zufallsname);
		rueck_string:=rueck_string:-append("'](width=5)");
		if j<nops(ListeSpaltenbezeichnungen) then
			rueck_string:=rueck_string:-append(",");
		elif j=nops(ListeSpaltenbezeichnungen) and i<nops(ListeZeilenbezeichnungen) then
			rueck_string:=rueck_string:-append("],\n");
		else	rueck_string:=rueck_string:-append("]]\n");
			rueck_string:=rueck_string:-append("\t\t),");
		end if;
	end do;
end do;	# schliesse grid


rueck_string:=rueck_string:-append("\n\t\tBoxRow(Button(\"Check Matrix\",\n\
\t\tEvaluate(function=\"check_matrix\",Argument('");
rueck_string:=rueck_string:-append(zufallsnamematrix);
rueck_string:=rueck_string:-append("'),Argument('");
rueck_string:=rueck_string:-append(zufallsnamezeilen);
rueck_string:=rueck_string:-append("'),Argument('");
rueck_string:=rueck_string:-append(zufallsnamespalten);
rueck_string:=rueck_string:-append("'))),\n\t\t");
rueck_string:=rueck_string:-append("Button(\"Zeige korrekte Matrix\",\n\t\t\
Evaluate(function=\"zeige_korrekte_matrix\",Argument('");
rueck_string:=rueck_string:-append(zufallsnamematrix);
rueck_string:=rueck_string:-append("'),Argument('");
rueck_string:=rueck_string:-append(zufallsnamezeilen);
rueck_string:=rueck_string:-append("'),Argument('");
rueck_string:=rueck_string:-append(zufallsnamespalten);
rueck_string:=rueck_string:-append("'))),\n\t\t");
rueck_string:=rueck_string:-append("Button(\"Lse System\",\n\t\t\
Evaluate(function=\"loese_matrixgleichung\",Argument('");
rueck_string:=rueck_string:-append(zufallsnamematrix);
rueck_string:=rueck_string:-append("'),Argument('");
rueck_string:=rueck_string:-append("spaltennamenfeld"):-append(zufallsname);
rueck_string:=rueck_string:-append("'))))\n\t\t");


rueck_string:=rueck_string:-append("\n\t\t]\n");

rueck_string:=rueck_string:-append("\t)\n");

return rueck_string:-value();

end:

################################################################################
#
# BoxRow fuer Foeppl-Funktionen
#
################################################################################

create_boxrow_foeppl:=proc(Ueberschrift, ListeFunktionsbezeichnungen, SchalterML, SchalterTool,\
				ListeNameKorrekteFunktionen)

#  Ueberschrift: String, der die Ueberschrift ueber die Ausdrucksbox enthaelt
#  ListeFunktionsbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 Namen der Funktionen enthlt (z.B. "Gleichgewicht in 
#				 x-Richtung").
#  SchalterML: boolean; true, falls Umwandlung in MathML-Schoenschrift angeboten werden soll
#	       sonst false.
#  SchalterTool:boolean; true, falls Button fuer Foeppl-Tool angeboten werden soll
#	       sonst false.
#  ListeNameKorrekteFunktionen: Liste mit Namen der korrekten Funktionen (wird zum Checken benoetigt).
#
# Rueckgabe: String mit entsprechender BoxRow-Spezifikation

local	rueck_string, i, funktionsfeld, \
	funktionsfeldname, zufallsname, mathmlfeld, mathmlfeldname;

rueck_string:=StringBuffer():

rueck_string:=rueck_string:-append("\tBoxRow( \n \t\t\
border=true, hscroll=as_needed,  vscroll=as_needed,\n");


for i from 1 to nops(ListeFunktionsbezeichnungen) do
zufallsname:=convert(rand(),string);
funktionsfeldname[i]:=cat("F",zufallsname,"name");
funktionsfeld[i]:=cat("F",zufallsname);
mathmlfeldname[i]:=cat("ML",zufallsname,"name");
mathmlfeld[i]:=cat("ML",zufallsname);
end do;


if length(Ueberschrift)>0 then
	rueck_string:=rueck_string:-append("\n\t\t[BoxRow(Label('font' = Font(\"arial\", bold, 12),\"");
	rueck_string:=rueck_string:-append(Ueberschrift);
	rueck_string:=rueck_string:-append("\")),");
else 	rueck_string:=rueck_string:-append("\n\t\t[");
end if;

if SchalterTool=true then
	rueck_string:=rueck_string:-append("BoxRow(\"Zum Experimentieren mit Foeppl-Funktionen \
knnen Sie ein Tool nutzen:\"\n\t\t,\
Button(\"Foeppl-Plot\", Evaluate(function='Maplets[Display](foeppl_plot_maplet)'))),");

end if;

for i from 1 to nops(ListeFunktionsbezeichnungen) do
	rueck_string:=rueck_string:-append("\n\t\tBoxRow(TextField['");
	rueck_string:=rueck_string:-append(funktionsfeldname[i]);
	rueck_string:=rueck_string:-append("'](value=\"");
	rueck_string:=rueck_string:-append(ListeNameKorrekteFunktionen[i]);
	rueck_string:=rueck_string:-append("\", visible=false,width=1),");

	rueck_string:=rueck_string:-append("\n\t\tTextField['");
	rueck_string:=rueck_string:-append(mathmlfeldname[i]);
	rueck_string:=rueck_string:-append("'](value=\"");
	rueck_string:=rueck_string:-append(mathmlfeld[i]);
	rueck_string:=rueck_string:-append("\", visible=false,width=1),\"");

	rueck_string:=rueck_string:-append(ListeFunktionsbezeichnungen[i]);
	rueck_string:=rueck_string:-append("\"\n\t\t, TextField['");
	rueck_string:=rueck_string:-append(ListeNameKorrekteFunktionen[i]);
	rueck_string:=rueck_string:-append("'](25, 'font' = Font(\"arial\",12)),");

	if SchalterML=true then
		rueck_string:=rueck_string:-append("\n\t\tButton(\">>schn>>\",  \n\t\t\t\
Evaluate(function=\"umwandeln\",Argument('");
		rueck_string:=rueck_string:-append(funktionsfeldname[i]);
		rueck_string:=rueck_string:-append("'),Argument('");
		rueck_string:=rueck_string:-append(mathmlfeldname[i]);
		rueck_string:=rueck_string:-append("'))),");
		rueck_string:=rueck_string:-append("\n\t\tMathMLViewer['");
		rueck_string:=rueck_string:-append(mathmlfeld[i]);
		rueck_string:=rueck_string:-append("'](height=50),\n");
	end if;
	rueck_string:=rueck_string:-append("\n\t\tButton(\"Check Funktion\",  \n\t\t\t\
Evaluate(function=\"check_foeppl\",Argument('");
	rueck_string:=rueck_string:-append(funktionsfeldname[i]);

	rueck_string:=rueck_string:-append("'))))");
	if i<nops(ListeFunktionsbezeichnungen) then
		rueck_string:=rueck_string:-append(",");
	else 	rueck_string:=rueck_string:-append("]");
	end if;
	rueck_string:=rueck_string:-append("\n\n");
end do;



rueck_string:=rueck_string:-append("\t)\n");

return rueck_string:-value();

end:



################################################################################
#
# Prozeduren zur Erzeugung des Aufgabenmapletkopfes und -endes 
# 
#
################################################################################


create_head:=proc(Mapletname, Titel, Beschreibungstext, Bilddatei, ZusatztextVariablenverwendung, Hilfetext)

# Mapletname: Name fuer das Aufgabenmaplet, Typ string
# Titel der Aufgabe, z.B. "Blatt 3, Aufgabe 4", Typ string
# Beschreibungstext:  Aufgabentext, Typ string
# Bilddatei:	Name der Bilddatei, falls ein Bild zur Aufgabenstellung gehoert, ansonsten, der leere
#		string ""; Typ string, Ordner mit vierfachem \ trennen(!), z.B. Bilder\\\\image01.gif
# ZusatztextVariablenverwendung: Text, der die zu benutzenden Variablennamen angibt und deren
#		Bedeutung angibt, Typ string
# Hilfetext: Text, der Hilfen zur Vorgehensweise gibt; Typ string

# Rueckgabe: String mit entsprechender Maplet-Spezifikation

local	rueck_string;

rueck_string:=StringBuffer():

rueck_string:=rueck_string:-append(Mapletname);
rueck_string:=rueck_string:-append(":=Maplet( 'onstartup'=RunWindow('W1'),\n\n\t");
rueck_string:=rueck_string:-append("Window['HText']('title'=\"Hilfe\", \n\t\t\
BoxRow(TextBox['T3'](width=40,height=4,editable=false,'font' = Font(\"arial\",12),\"");
rueck_string:=rueck_string:-append(Hilfetext);
rueck_string:=rueck_string:-append("\"), Button(\"OK\",CloseWindow('HText')))),\n\n\tWindow['W1'](");
 
rueck_string:=rueck_string:-append("'title'=\"");
rueck_string:=rueck_string:-append(Titel);
rueck_string:=rueck_string:-append("\",\n\tBoxRow('border'=false,hscroll=as_needed,  vscroll=as_needed,\n\n\t\t");
rueck_string:=rueck_string:-append("[BoxRow('border'=false,Label(\"");
rueck_string:=rueck_string:-append(Titel);
rueck_string:=rueck_string:-append(" \",'halign'=right, 'font' = Font(\"arial\", bold, 16))),\n\t\t");

rueck_string:=rueck_string:-append("BoxRow('border'=true,[	\n\t\t\tTextBox['T1'](\
width=20,height=2,editable=false,'font' = Font(\"arial\",12),\"");
rueck_string:=rueck_string:-append(Beschreibungstext);
rueck_string:=rueck_string:-append("\")],\n\t\t\t[Label(Image(\"");
rueck_string:=rueck_string:-append(Bilddatei);
rueck_string:=rueck_string:-append("\"))]),\n");


rueck_string:=rueck_string:-append("\t\tBoxRow('border'=true, [[Label(\"Bitte verwenden Sie folgende Variablennamen:\",'font' = Font(\"arial\", bold, 12))],[	\n\t\t\tTextBox['T2'](\
width=20,height=5,editable=false,'font' = Font(\"arial\",12),\"");
rueck_string:=rueck_string:-append(ZusatztextVariablenverwendung);
rueck_string:=rueck_string:-append("\")]]),\n");

rueck_string:=rueck_string:-append("\t\tBoxRow('border'=true,");
rueck_string:=rueck_string:-append("[\"Bentigen Sie eine Hilfe zur Vorgehensweise?\"],\n\t\t\t");
rueck_string:=rueck_string:-append("Button(\"Hilfe\", RunWindow('HText') ))\n\n");

#### folgende Zeile nur zu Testzwecken, spaeter entfernen!

#rueck_string:=rueck_string:-append("\t\t]\n\t)\n\t)\n):");

return rueck_string:-value();

end:


create_tail:=proc(Loesungsdatei)

# Loesungsdatei: Name der PDF-Datei (Muss PDF sein!), die die Loesung enthlt

# Rueckgabe: String mit entsprechender Maplet-Spezifikation

local	rueck_string;

rueck_string:=StringBuffer():
rueck_string:=rueck_string:-append("\n\t\tBoxRow('border'=false,\n\t\t\t");
rueck_string:=rueck_string:-append("TextField['Loesungsdatei'](value=\"");
rueck_string:=rueck_string:-append(Loesungsdatei);
rueck_string:=rueck_string:-append("\", visible=false,width=1),\n");


rueck_string:=rueck_string:-append("\t\t[Button(\"Lsung\", Evaluate(function=\"zeige_loesung\",Argument('Loesungsdatei')))],\
[Button(\"Zurck zur Auswahl\", Shutdown())])");

rueck_string:=rueck_string:-append("\n\n\t\t]\n\t)\n\t)\n):");

return rueck_string:-value();

end:


################################################################################
#
# Prozeduren zur Erstellung eines Aufgabenmaplets 
# 
#
################################################################################

create_exercise_maplet:=proc(Dateiname, Schreibmodus, Mapletname, Titel, Aufgabentext, Bilddatei, \
ZusatztextVariablenverwendung, Hilfetext, Loesungsdateiname, Checkboxliste)

# Dateiname: Name der Datei, in der das Maplet gespeichert wird; Typ string
# Schreibmodus: gibt an, ob Maplet an die bestehende Datei angehaengt werden soll
#		oder ob diese ueberschrieben werden soll; Angabe: APPEND oder WRITE 
# Mapletname: Name fuer das Aufgabenmaplet, Typ string
# Titel der Aufgabe, z.B. "Blatt 3, Aufgabe 4", Typ string
# Aufgabentext:  Aufgabentext, Typ string
# Bilddatei:	Name der Bilddatei, falls ein Bild zur Aufgabenstellung gehoert, ansonsten, der leere
#		string ""; Typ string, Ordner mit vierfachem \ trennen(!), z.B. Bilder\\\\image01.gif
# ZusatztextVariablenverwendung: Text, der die zu benutzenden Variablennamen angibt und deren
#		Bedeutung angibt, Typ string
# Hilfetext: Text, der Hilfen zur Vorgehensweise gibt; Typ string
# Loesungsdateiname: Name der PDF-Datei (Muss PDF sein!), die die Loesung enthlt
# Checkboxliste: Liste der einzufuegenden Boxen zum Checken mathematischer Objekte
#		 Jeder Listeneintrag ist wiederum eine Liste, die folgende Gestalt haben muss:
#
#		 bei linearen Gleichungssystemen: [LGS, Ueberschrift, ListeGleichungsbezeichnungen, 
#							SchalterML, NameKorrektesSystem]
#			Bedeutung: LGS: Der erste Listeneintrag LGS zeigt, dass es sich um ein LGS handelt.
#  				   Ueberschrift: String, der die Ueberschrift ueber die Gleichungsbox enthaelt
#  				   ListeGleichungsbezeichnungen: Liste bestehend aus Strings, die die 
#								im Maplet erscheinenden Namen der 
#				 				Gleichungen enthlt (z.B. "Gleichgewicht in 
#				 				x-Richtung"); soll keine Name erscheinen, 
#				 				wird ein leerer String ("") verwendet.
#  				   SchalterML: boolean; true, falls Umwandlung in MathML-Schoenschrift 
#							angeboten werden soll, sonst false.	       
#  				   NameKorrektesSystem: Name deskorrekten Systems (wird zum Checken benoetigt).
#
#		bei Ausdruecken: [AUSDRUECKE, Ueberschrift, ListeAusdrucksbezeichnungen, SchalterML, 
#				    ListeNameKorrekteAusdruecke]
#			Bedeutung: AUSDRUECKE: Der erste Listeneintrag AUSDRUECKE zeigt, dass es sich um Ausdruecke handelt.
#  				   Ueberschrift: String, der die Ueberschrift ueber die Ausdrucksbox enthaelt
#  				   ListeAusdrucksbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 		Namen der Ausdruecke enthlt (z.B. "Gleichgewicht in 
#				 		x-Richtung").
#  				   SchalterML: boolean; true, falls Umwandlung in MathML-Schoenschrift angeboten werden soll
#	       					sonst false.
#  				   ListeNameKorrekteAusdruecke: Liste mit Namen der korrekten Ausdruecke (wird zum Checken benoetigt).
#
#		bei Vektoren: [VEKTOREN, Ueberschrift, ListeVektorbezeichnungen, SchalterML,
#				ListeNameKorrekteVektoren]
#			Bedeutung: VEKTOREN: Der erste Listeneintrag VEKTOREN zeigt, dass es sich um Vektoren handelt.
#  				   Ueberschrift: String, der die Ueberschrift ueber die Vektorenbox enthaelt
#  				   ListeVektorbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 		Namen der Vektoren enthlt (z.B. "F").
#  				   SchalterML: boolean; true, falls Umwandlung in MathML-Schoenschrift angeboten werden soll
#	       					sonst false. Wird momentan nicht angeboten.
#  				   ListeNameKorrekteVektoren: Liste mit Namen der korrekten Vektoren (wird zum Checken benoetigt).
#
#		bei einer Matrix: [MATRIX, Ueberschrift, ListeZeilenbezeichnungen, ListeSpaltenbezeichnungen, SchalterML, \
#					NameKorrekteMatrix]
#			Bedeutung: MATRIX: Der erste Listeneintrag MATRIX zeigt, dass es sich um eine Matrix handelt.
#  				   Ueberschrift: String, der die Ueberschrift ueber die Matrixbox enthaelt
#  				   ListeZeilenbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 		Namen der Zeilen enthlt (z.B. "F").
#  				   ListeSpaltenbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 		Namen der Spalten enthlt (z.B. "F").
#  				   SchalterML: boolean; true, falls Umwandlung in MathML-Schoenschrift angeboten werden soll
#	       				sonst false. Wird momentan nicht angeboten.
#  				   NameKorrekteMatrix: String, der den Namen der korrekten Matrix enthaelt (wird zum Checken benoetigt).
#
#		bei einer Foeppl-Funktion [FOEPPL, Ueberschrift, ListeFunktionsbezeichnungen, SchalterML, SchalterTool,
#				ListeNameKorrekteFunktionen]
#			Bedeutung: FOEPPL: Der erste Listeneintrag FOEPPL zeigt, dass es sich um eine Foeppl-Funktion handelt.
#  				   Ueberschrift: String, der die Ueberschrift ueber die Ausdrucksbox enthaelt
#  				   ListeFunktionsbezeichnungen: Liste bestehend aus Strings, die die im Maplet erscheinenden
#				 	Namen der Funktionen enthlt (z.B. "Gleichgewicht in 
#				 	x-Richtung").
#  				   SchalterML: boolean; true, falls Umwandlung in MathML-Schoenschrift angeboten werden soll
#	       				sonst false.
#  				   SchalterTool:boolean; true, falls Button fuer Foeppl-Tool angeboten werden soll
#	       				sonst false.
#  				   ListeNameKorrekteFunktionen: Liste mit Namen der korrekten Funktionen (wird zum Checken benoetigt).
#
#
# Rueckgabe: 0, falls alles geklappt hat, sonst 1.

local	fd, textstring, i;

fd:=fopen(Dateiname, Schreibmodus);

textstring:=StringBuffer();

textstring:=textstring:-append(create_head(Mapletname, Titel, Aufgabentext, Bilddatei,\
				ZusatztextVariablenverwendung, Hilfetext));

textstring:=textstring:-append(",\n\n");

textstring:=textstring:-append("\t######################################################################\n\n");

textstring:=textstring:-append("\t#     Beginn der Checkboxen\n\n");

textstring:=textstring:-append("\t######################################################################\n\n");

for i from 1 to nops(Checkboxliste) do

	if Checkboxliste[i][1]=LGS then
		textstring:=textstring:-append("\t######################################################################\n\n");
		textstring:=textstring:-append("\t#     Checkbox fuer LGS\n\n");
		textstring:=textstring:-append("\t######################################################################\n\n");
		textstring:=textstring:-append(create_boxrow_lgs(Checkboxliste[i][2],Checkboxliste[i][3],\
							Checkboxliste[i][4],Checkboxliste[i][5])):-append(",\n");
		textstring:=textstring:-append("\t#######  Ende Checkbox LGS     ####################\n\n");

	elif Checkboxliste[i][1]=AUSDRUECKE then
		textstring:=textstring:-append("\t######################################################################\n\n");
		textstring:=textstring:-append("\t#     Checkbox fuer Ausdruecke\n\n");
		textstring:=textstring:-append("\t######################################################################\n\n");
		textstring:=textstring:-append(create_boxrow_ausdruecke(Checkboxliste[i][2],Checkboxliste[i][3],\
							Checkboxliste[i][4],Checkboxliste[i][5])):-append(",\n");
		textstring:=textstring:-append("\t#######  Ende Checkbox Ausdruecke     ####################\n\n");


	elif Checkboxliste[i][1]=VEKTOREN then
		textstring:=textstring:-append("\t######################################################################\n\n");
		textstring:=textstring:-append("\t#     Checkbox fuer Vektoren\n\n");
		textstring:=textstring:-append("\t######################################################################\n\n");
		textstring:=textstring:-append(create_boxrow_vektoren(Checkboxliste[i][2],Checkboxliste[i][3],\
							Checkboxliste[i][4],Checkboxliste[i][5])):-append(",\n");
		textstring:=textstring:-append("\t#######  Ende Checkbox Vektoren     ####################\n\n");


	elif Checkboxliste[i][1]=MATRIX then
		textstring:=textstring:-append("\t######################################################################\n\n");
		textstring:=textstring:-append("\t#     Checkbox fuer Matrix\n\n");
		textstring:=textstring:-append("\t######################################################################\n\n");
		textstring:=textstring:-append(create_boxrow_matrix(Checkboxliste[i][2],Checkboxliste[i][3],\
							Checkboxliste[i][4],Checkboxliste[i][5],Checkboxliste[i][6])):-append(",\n");
		textstring:=textstring:-append("\t#######  Ende Checkbox Matrix     ####################\n\n");

	elif Checkboxliste[i][1]=FOEPPL then
		textstring:=textstring:-append("\t######################################################################\n\n");
		textstring:=textstring:-append("\t#     Checkbox fuer Foeppl-Funktionen\n\n");
		textstring:=textstring:-append("\t######################################################################\n\n");
		textstring:=textstring:-append(create_boxrow_foeppl(Checkboxliste[i][2],Checkboxliste[i][3],\
							Checkboxliste[i][4],Checkboxliste[i][5],Checkboxliste[i][6])):-append(",\n");
		textstring:=textstring:-append("\t#######  Ende Checkbox Foeppl-Funktionen     ####################\n\n");

	else print("fehlerhafte Objektangabe:", Checkboxliste[i][1],Checkboxliste[i],Mapletname);
	end if;
end do;

textstring:=textstring:-append(create_tail(Loesungsdateiname));

textstring:=textstring:-append("\n\n#################################################################"):
textstring:=textstring:-append("\n\n######    Ende Maplet "):
textstring:=textstring:-append(Mapletname);
textstring:=textstring:-append("     ################");
textstring:=textstring:-append("\n\n#################################################################\n\n"):


fprintf(fd,textstring:-value());
fclose(fd);

return 0;

end:

